@@ -25,8 +25,10 @@ An emoji picker for ibus typing booster.
 from typing import Any
 from typing import List
 from typing import Tuple
+from typing import Iterable
 from typing import Dict
 from typing import Optional
+from typing import Callable
 import sys
 import os
 import re
@@ -39,8 +41,6 @@ import unicodedata
 import html
 import logging
 import logging.handlers
-import itb_util
-import itb_pango
 
 from gi import require_version # type: ignore
 require_version('GLib', '2.0')
@@ -58,6 +58,8 @@ require_version('Gtk', '3.0')
 from gi.repository import Gtk
 from gi.repository import GObject
 import itb_emoji
+import itb_util
+import itb_pango
 import itb_version
 
 LOGGER = logging.getLogger('ibus-typing-booster')
@@ -67,8 +69,8 @@ GTK_VERSION = (Gtk.get_major_version(),
                Gtk.get_micro_version())
 
 DOMAINNAME = 'ibus-typing-booster'
-_ = lambda a: gettext.dgettext(DOMAINNAME, a)
-N_ = lambda a: a
+_: Callable[[str], str] = lambda a: gettext.dgettext(DOMAINNAME, a)
+N_: Callable[[str], str] = lambda a: a
 
 def parse_args() -> Any:
     '''
@@ -83,11 +85,11 @@ def parse_args() -> Any:
         action='store',
         default='',
         help=('Set a list of languages to be used when browsing '
-              + 'or searching for emoji. For example: '
-              + '"emoji-picker -l de:fr:ja" '
-              + 'would use German, French, and Japanese. '
-              + 'If empty, the locale settings are used to '
-              + 'determine the languages.'))
+              'or searching for emoji. For example: '
+              '"emoji-picker -l de:fr:ja" '
+              'would use German, French, and Japanese. '
+              'If empty, the locale settings are used to '
+              'determine the languages.'))
     parser.add_argument(
         '-f', '--font',
         nargs='?',
@@ -95,9 +97,9 @@ def parse_args() -> Any:
         action='store',
         default=None,
         help=('Set a font to display emoji. '
-              + 'If not specified, the font is read from the config file. '
-              + 'To use the system default font specify "emoji". '
-              + 'default: "%(default)s"'))
+              'If not specified, the font is read from the config file. '
+              'To use the system default font specify "emoji". '
+              'default: "%(default)s"'))
     parser.add_argument(
         '-s', '--fontsize',
         nargs='?',
@@ -105,24 +107,24 @@ def parse_args() -> Any:
         action='store',
         default=None,
         help=('Set a fontsize to display emoji. '
-              + 'If not specified, the fontsize is read from the config file. '
-              + 'If that fails 24 is used as a fall back fontsize. '
-              + 'default: "%(default)s"'))
+              'If not specified, the fontsize is read from the config file. '
+              'If that fails 24 is used as a fall back fontsize. '
+              'default: "%(default)s"'))
     parser.add_argument(
         '-m', '--modal',
         action='store_true',
         default=False,
         help=('Make the window of emoji-picker modal. '
-              + 'default: %(default)s'))
+              'default: %(default)s'))
     parser.add_argument(
         '-a', '--all',
         action='store_true',
         default=False,
         help=('Load all Unicode characters. '
-              + 'Makes all Unicode characters accessible, '
-              + 'even normal letters. '
-              + 'Slows the search down and is usually not needed. '
-              + 'default: %(default)s'))
+              'Makes all Unicode characters accessible, '
+              'even normal letters. '
+              'Slows the search down and is usually not needed. '
+              'default: %(default)s'))
     parser.add_argument(
         '--fallback',
         nargs='?',
@@ -130,12 +132,12 @@ def parse_args() -> Any:
         action='store',
         default=None,
         help=('Whether to use fallback fonts when rendering emoji. '
-              + 'If not 0, pango will use fallback fonts as necessary. '
-              + 'If 0, pango will use only glyphs from '
-              + 'the closest matching font on the system. No fallback '
-              + 'will be done to other fonts on the system that '
-              + 'might contain the glyhps needed to render an emoji. '
-              + 'default: "%(default)s"')
+              'If not 0, pango will use fallback fonts as necessary. '
+              'If 0, pango will use only glyphs from '
+              'the closest matching font on the system. No fallback '
+              'will be done to other fonts on the system that '
+              'might contain the glyhps needed to render an emoji. '
+              'default: "%(default)s"')
         )
     parser.add_argument(
         '--emoji_unicode_min',
@@ -144,8 +146,8 @@ def parse_args() -> Any:
         action='store',
         default='0.0',
         help=('Load only emoji which were added to Unicode '
-              + 'not earlier than this Unicode version. '
-              + 'default: %(default)s'))
+              'not earlier than this Unicode version. '
+              'default: %(default)s'))
     # Guess current Unicode version:
     unicode_versions = (
         ('20180605', '11.0'),
@@ -153,6 +155,7 @@ def parse_args() -> Any:
         ('20190507', '12.1'),
         ('20200701', '13.0'),
         ('20200915', '13.1'),
+        ('20210914', '14.0'),
     )
     current_date = time.strftime('%Y%m%d')
     current_unicode_version = '13.0'
@@ -166,37 +169,37 @@ def parse_args() -> Any:
         action='store',
         default=current_unicode_version,
         help=('Load only emoji which were added to Unicode '
-              + 'not later than this Unicode version. '
-              + 'default: %(default)s'))
+              'not later than this Unicode version. '
+              'default: %(default)s'))
     parser.add_argument(
         '-d', '--debug',
         action='store_true',
         default=False,
         help=('Print some debug output to stdout. '
-              + 'default: %(default)s'))
+              'default: %(default)s'))
     parser.add_argument(
         '--version',
         action='store_true',
         default=False,
         help=('Output version information and exit. '
-              + 'default: %(default)s'))
+              'default: %(default)s'))
     return parser.parse_args()
 
 _ARGS = parse_args()
 
-class EmojiPickerUI(Gtk.Window):
+class EmojiPickerUI(Gtk.Window): # type: ignore
     '''
     User interface of the emoji picker
     '''
     def __init__(self,
-                 languages=('en_US',),
-                 modal=False,
-                 unicode_data_all=False,
-                 emoji_unicode_min=1.0,
-                 emoji_unicode_max=100.0,
-                 font=None,
-                 fontsize=None,
-                 fallback=None) -> None:
+                 languages: Iterable[str] = ('en_US',),
+                 modal: bool = False,
+                 unicode_data_all: bool = False,
+                 emoji_unicode_min: str = '1.0',
+                 emoji_unicode_max: str = '100.0',
+                 font: Optional[str] = None,
+                 fontsize: Optional[float] = None,
+                 fallback: Optional[bool] = None) -> None:
         Gtk.Window.__init__(self, title='🚀 ' + _('Emoji Picker'))
 
         self.set_name('EmojiPicker')
@@ -235,7 +238,7 @@ class EmojiPickerUI(Gtk.Window):
         #
         # It only works like this when gnome-shell runs under Xorg
         # though, under Wayland things are different.
-        self.set_wmclass('emoji-picker', 'Emoji Picker')
+        self.set_wmclass('emoji-picker', 'Emoji Picker') # pylint: disable=no-member
         self.set_default_size(700, 400)
         self._modal = modal
         self.set_modal(self._modal)
@@ -301,7 +304,7 @@ class EmojiPickerUI(Gtk.Window):
         self._skin_tone_selected_popover: Optional[Gtk.Popover] = None
 
         self._main_container = Gtk.VBox()
-        self.add(self._main_container)
+        self.add(self._main_container) # pylint: disable=no-member
         self._header_bar = Gtk.HeaderBar()
         self._header_bar.set_hexpand(True)
         self._header_bar.set_vexpand(False)
@@ -430,7 +433,8 @@ class EmojiPickerUI(Gtk.Window):
         self._flowbox_scroll.set_overlay_scrolling(True)
         self._flowbox = Gtk.FlowBox()
         self._flowbox_scroll.add(self._flowbox)
-        self._long_press_gestures: List[Tuple[Any, Tuple[Any, Any, Any, Any]]] = []
+        self._long_press_gestures: List[
+            Tuple[Any, Tuple[Any, Any, Any, Any]]] = []
 
         self._left_pane = Gtk.VBox()
         self._left_pane.set_homogeneous(False)
@@ -501,11 +505,11 @@ class EmojiPickerUI(Gtk.Window):
         if _ARGS.debug:
             LOGGER.debug(
                 'expanded_languages  = %s\n'
-                %itb_util.expand_languages(self._languages)
-                + 'first_language_with_categories = %s\n'
-                %first_language_with_categories
-                + 'number_of_empty_languages = %s\n'
-                %number_of_empty_languages)
+                'first_language_with_categories = %s\n'
+                'number_of_empty_languages = %s\n',
+                itb_util.expand_languages(self._languages),
+                first_language_with_categories,
+                number_of_empty_languages)
 
         self._browse_treeview.append_column(
             Gtk.TreeViewColumn(
@@ -525,7 +529,7 @@ class EmojiPickerUI(Gtk.Window):
 
         self._browse_treeview.columns_autosize()
 
-        self.show_all()
+        self.show_all() # pylint: disable=no-member
 
         (dummy_minimum_width_search_entry,
          natural_width_search_entry) = self._search_entry.get_preferred_width()
@@ -632,7 +636,7 @@ class EmojiPickerUI(Gtk.Window):
             N_('Keywords'),
         ]
         if self._gettext_translations[language]:
-            return self._gettext_translations[language].gettext(key)
+            return str(self._gettext_translations[language].gettext(key))
         return key
 
     def _emoji_descriptions(self, emoji: str) -> List[str]:
@@ -867,7 +871,7 @@ class EmojiPickerUI(Gtk.Window):
         for child in self._flowbox.get_children():
             child.set_can_focus(False)
 
-        self.show_all()
+        self.show_all() # pylint: disable=no-member
         self._busy_stop()
 
     def _read_options(self) -> None:
@@ -877,10 +881,10 @@ class EmojiPickerUI(Gtk.Window):
         options_dict = {}
         if os.path.isfile(self._options_file):
             try:
-                options_dict = eval(open(
-                    self._options_file,
-                    mode='r',
-                    encoding='UTF-8').read())
+                with open(self._options_file,
+                          mode='r',
+                          encoding='UTF-8') as options_file:
+                    options_dict = eval(options_file.read())
             except (PermissionError, SyntaxError, IndentationError):
                 LOGGER.exception('Error when reading options')
             except Exception as dummy_exception:
@@ -940,10 +944,10 @@ class EmojiPickerUI(Gtk.Window):
         recently_used_emoji = {}
         if os.path.isfile(self._recently_used_emoji_file):
             try:
-                recently_used_emoji = eval(open(
-                    self._recently_used_emoji_file,
-                    mode='r',
-                    encoding='UTF-8').read())
+                with open(self._recently_used_emoji_file,
+                          mode='r',
+                          encoding='UTF-8') as recently_used_file:
+                    recently_used_emoji = eval(recently_used_file.read())
             except (PermissionError, SyntaxError, IndentationError):
                 import traceback
                 traceback.print_exc()
@@ -972,7 +976,7 @@ class EmojiPickerUI(Gtk.Window):
             self._init_recently_used()
         self._cleanup_recently_used()
 
-    def _cleanup_recently_used(self):
+    def _cleanup_recently_used(self) -> None:
         '''
         Reduces the size of the recently used dictionary
         if it has become too big.
@@ -1000,10 +1004,10 @@ class EmojiPickerUI(Gtk.Window):
                 ('', '', self._recently_used_label)):
             self._clear_flowbox()
 
-    def on_clear_recently_used_button_clicked(self, _button) -> None:
+    def on_clear_recently_used_button_clicked(
+            self, _button: Gtk.Button) -> None:
         '''
         :param _button: The “Clear recently used” button
-        :type _button: Gtk.Button object
         '''
         if _ARGS.debug:
             LOGGER.debug('on_clear_recently_used_button_clicked()\n')
@@ -1012,12 +1016,11 @@ class EmojiPickerUI(Gtk.Window):
         self._main_menu_popover.hide()
         self._init_recently_used()
 
-    def _add_to_recently_used(self, emoji) -> None:
+    def _add_to_recently_used(self, emoji: str) -> None:
         '''
         Adds an emoji to the resently used dictionary.
 
         :param emoji: The emoji string
-        :type emoji: String
         '''
         if emoji in self._recently_used_emoji:
             self._recently_used_emoji[emoji]['count'] += 1
@@ -1053,7 +1056,8 @@ class EmojiPickerUI(Gtk.Window):
         self._selection_clipboard.store()
         self._selection_primary.store() # Does not work.
 
-    def _show_concise_emoji_description_in_header_bar(self, emoji: str) -> None:
+    def _show_concise_emoji_description_in_header_bar(
+            self, emoji: str) -> None:
         '''
         Show a concise description of an emoji in the header bar
         of the program.
@@ -1093,7 +1097,7 @@ class EmojiPickerUI(Gtk.Window):
         stats.print_stats('itb_emoji', 25)
         LOGGER.info('Profiling info:\n%s', stats_stream.getvalue())
 
-    def on_delete_event(self, *_args) -> None:
+    def on_delete_event(self, *_args: Any) -> None:
         '''
         The window has been deleted, probably by the window manager.
         '''
@@ -1102,7 +1106,7 @@ class EmojiPickerUI(Gtk.Window):
             self._print_profiling_information()
         Gtk.main_quit()
 
-    def on_destroy_event(self, *_args) -> None:
+    def on_destroy_event(self, *_args: Any) -> None:
         '''
         The window has been destroyed.
         '''
@@ -1172,12 +1176,11 @@ class EmojiPickerUI(Gtk.Window):
                 Gtk.main_iteration()
         self._search_bar.handle_event(event_key)
 
-    def on_about_button_clicked(self, _button) -> None:
+    def on_about_button_clicked(self, _button: Gtk.Button) -> None:
         '''
         The “About” button has been clicked
 
         :param _button: The “About” button
-        :type _button: Gtk.Button object
         '''
         if _ARGS.debug:
             LOGGER.debug('on_about_button_clicked()\n')
@@ -1287,18 +1290,17 @@ class EmojiPickerUI(Gtk.Window):
         for child in self._flowbox.get_children():
             child.set_can_focus(False)
 
-        self.show_all()
+        self.show_all() # pylint: disable=no-member
         self._candidates_invalid = False
         self._busy_stop()
 
     def on_fontsize_spin_button_grab_focus( # pylint: disable=no-self-use
-            self, spin_button) -> bool:
+            self, spin_button: Gtk.SpinButton) -> bool:
         '''
         Signal handler called when the spin button to change
         the font size grabs focus
 
         :param spin_button: The spin button to change the font size
-        :type spin_button: Gtk.SpinButton object
         '''
         if _ARGS.debug:
             LOGGER.debug(
@@ -1312,12 +1314,11 @@ class EmojiPickerUI(Gtk.Window):
         return True
 
     def on_search_entry_grab_focus( # pylint: disable=no-self-use
-            self, search_entry) -> bool:
+            self, search_entry: Gtk.SearchEntry) -> bool:
         '''
         Signal handler called when the search entry grabs focus
 
         :param search_entry: The search entry
-        :type search_entry: Gtk.SearchEntry object
         '''
         if _ARGS.debug:
             LOGGER.debug(
@@ -1440,7 +1441,7 @@ class EmojiPickerUI(Gtk.Window):
             self._emoji_selected_popover = None
         return False
 
-    def _emoji_event_box_selected(self, event_box): # FIXME add type hint
+    def _emoji_event_box_selected(self, event_box: Gtk.EventBox) -> bool:
         '''
         Called when an event box containing an emoji
         was selected in the flowbox.
@@ -1450,7 +1451,6 @@ class EmojiPickerUI(Gtk.Window):
         has been copied to the clipboard.
 
         :param event_box: The event box which contains the emoji
-        :type event_box: Gtk.EventBox object
         '''
         # Use .get_label() instead of .get_text() to fetch the text
         # from the label widget including any embedded underlines
@@ -1462,7 +1462,8 @@ class EmojiPickerUI(Gtk.Window):
             LOGGER.debug('text = %s\n', text)
         (emoji, name) = self._parse_emoji_and_name_from_text(text)
         if not emoji:
-            return Gdk.EVENT_PROPAGATE
+            # Gdk.EVENT_PROPAGATE is defined as False
+            return bool(Gdk.EVENT_PROPAGATE)
         self._show_concise_emoji_description_in_header_bar(emoji)
         self._set_clipboards(emoji)
         self._add_to_recently_used(emoji)
@@ -1483,6 +1484,7 @@ class EmojiPickerUI(Gtk.Window):
             self._emoji_selected_popover.popup()
         self._emoji_selected_popover.show_all()
         GLib.timeout_add(500, self._emoji_selected_popover_popdown)
+        return True # Gdk.EVENT_PROPAGATE is defined as False
 
     def on_emoji_selected(
             self,
@@ -1552,7 +1554,7 @@ class EmojiPickerUI(Gtk.Window):
                         + '</span>')
                 label.set_text(new_text)
                 label.set_use_markup(True)
-        self.show_all()
+        self.show_all() # pylint: disable=no-member
         self._busy_stop()
 
     def _skin_tone_selected_popover_popdown(self) -> bool:
@@ -1640,9 +1642,7 @@ class EmojiPickerUI(Gtk.Window):
         Signal handler called when the gesture is recognized
 
         :param gesture: The object which received the signal
-        :type gesture: Gtk.GestureLongPress object
         :param event_sequence: the event sequence that the event belongs to
-        :type event_sequence: Gdk.EventSequence structure
         '''
         if _ARGS.debug:
             LOGGER.debug(
@@ -1680,20 +1680,21 @@ class EmojiPickerUI(Gtk.Window):
     def on_flowbox_event_box_long_press_pressed(
             self,
             gesture: Gtk.GestureLongPress,
-            x: int,
-            y: int,
+            x_coordinate: int,
+            y_coordinate: int,
             event_box: Gtk.EventBox) -> None:
         '''
         :param gesture: The object which received the signal
-        :param x: the X coordinate where the press happened,
-                  relative to the widget allocation
-        :param y: the Y coordinate where the press happened,
-                  relative to the widget allocation
+        :param x_coordinate: the X coordinate where the press happened,
+                             relative to the widget allocation
+        :param y_coordinate: the Y coordinate where the press happened,
+                             relative to the widget allocation
         '''
         if _ARGS.debug:
             LOGGER.debug(
                 'on_flowbox_event_box_long_press_pressed() %s %s %s %s\n',
-                repr(gesture), x, y, repr(event_box))
+                repr(gesture),
+                x_coordinate, y_coordinate, repr(event_box))
         self._show_skin_tone_popover(event_box)
 
     def on_flowbox_event_box_button_release(
@@ -1740,7 +1741,6 @@ class EmojiPickerUI(Gtk.Window):
 
         :param event_box: The event box containing the label with the emoji.
                           The popover will be relative to this event box.
-        :type event_box: Gtk.EventBox object
         '''
         text = event_box.get_child().get_label()
         (emoji, name) = self._parse_emoji_and_name_from_text(text)
@@ -2026,7 +2026,6 @@ class EmojiPickerUI(Gtk.Window):
 
         :param toggle_button: The check button used to select whether
                               fallback fonts should be used.
-        :type adjustment: Gtk.CheckButton object
         '''
         self._fallback = check_button.get_active()
         if _ARGS.debug:
@@ -2040,8 +2039,6 @@ class EmojiPickerUI(Gtk.Window):
     def _list_font_names(self) -> List[str]:
         '''
         Returns a list of font names available on the system
-
-        :rtype: List of strings
         '''
         good_emoji_fonts = [
             'Noto Color Emoji 🎨',
@@ -2066,7 +2063,7 @@ class EmojiPickerUI(Gtk.Window):
         families = pango_context.list_families()
         names = sorted([family.get_name() for family in families])
         for font in good_emoji_fonts:
-            name = font.split(' ')[0]
+            name = font.split(' ', maxsplit=1)[0]
             if name in names:
                 names.remove(name)
                 available_good_emoji_fonts.append(font)
@@ -2225,7 +2222,7 @@ def get_languages() -> List[str]:
     If not, get it from the environment variables.
     '''
     if _ARGS.languages:
-        return _ARGS.languages.split(':')
+        return list(_ARGS.languages.split(':'))
     environment_variable = os.getenv('LANGUAGE')
     if not environment_variable:
         environment_variable = os.getenv('LC_ALL')
