@@ -27,12 +27,25 @@ hunspell dictonaries.
 from typing import Dict
 from typing import Tuple
 from typing import List
+from typing import Iterable
 import os
 import sys
 import unicodedata
 import logging
 import itb_util
 
+IMPORT_REGEX_SUCCESFUL = False
+try:
+    import regex # type: ignore
+    IMPORT_REGEX_SUCCESFUL = True
+    # Enable new improved regex engine instead of backwards compatible
+    # v0.  regex.match('ß', 'SS', regex.IGNORECASE) matches only with
+    # the improved version!  See also: https://pypi.org/project/regex/
+    regex.DEFAULT_VERSION = regex.VERSION1
+except (ImportError,):
+    # Use standard “re” module as a fallback:
+    import re
+
 LOGGER = logging.getLogger('ibus-typing-booster')
 
 DEBUG_LEVEL = int(0)
@@ -67,7 +80,7 @@ MAX_WORDS = 100
 class Dictionary:
     '''A class to hold a hunspell dictionary
     '''
-    def __init__(self, name='en_US') -> None:
+    def __init__(self, name: str = 'en_US') -> None:
         if DEBUG_LEVEL > 1:
             LOGGER.debug('Dictionary.__init__(name=%s)\n', name)
         self.name = name
@@ -181,16 +194,14 @@ class Dictionary:
         if not self.voikko:
             return False
         # voikko works correctly if the input is a Unicode string in NFC.
-        return self.voikko.spell(unicodedata.normalize('NFC', word))
+        return bool(self.voikko.spell(unicodedata.normalize('NFC', word)))
 
     def spellcheck(self, word: str) -> bool:
         '''
         Spellcheck a word using enchant, pyhunspell, or voikko
 
         :param word: The word to spellcheck
-        :type word: String
         :return: True if spelling is correct, False if not or unknown
-        :rtype: Boolean
 
         >>> d = Dictionary('en_US')
         >>> d.spellcheck('winter')
@@ -211,7 +222,7 @@ class Dictionary:
         if self.pyhunspell_object:
             return self.spellcheck_pyhunspell(word)
         if self.voikko:
-            return self.voikko.spell(word)
+            return bool(self.voikko.spell(word))
         return False
 
     def has_spellchecking(self) -> bool:
@@ -219,7 +230,6 @@ class Dictionary:
         Returns wether this dictionary supports spellchecking or not
 
         :return: True if this dictionary spports spellchecking, False if not
-        :rtype: Boolean
 
         Examples:
 
@@ -295,19 +305,22 @@ class Dictionary:
             ]
 
     def spellcheck_suggest(self, word: str) -> List[str]:
-        '''
-        Return spellchecking suggestions for word using enchant, pyhunspell or voikko
+        '''Return spellchecking suggestions for word using enchant,
+        pyhunspell or voikko
 
         :param word: The word to return spellchecking suggestions for
-        :type word: String
         :return: List of spellchecking suggestions, possibly empty.
-        :rtype: List of strings
+
+        Results can be quite different depending on whether enchant or
+        pyhunspell is used and in case of enchant whether hunspell,
+        aspell, myspell, or ispell are used. So for the doctests,
+        better don’t check the complete list of suggestions returned.
 
         Examples:
 
         >>> d = Dictionary('en_US')
-        >>> d.spellcheck_suggest('kamel')
-        ['camel', 'Camel']
+        >>> 'Camel' in d.spellcheck_suggest('kamel')
+        True
 
         >>> d.spellcheck_suggest('')
         []
@@ -315,6 +328,7 @@ class Dictionary:
         >>> d = Dictionary('None')
         >>> d.spellcheck_suggest('kamel')
         []
+
         '''
         if self.enchant_dict:
             return self.spellcheck_suggest_enchant(word)
@@ -328,10 +342,11 @@ class Hunspell:
     '''A class to suggest completions or corrections
     using a list of Hunspell dictionaries
     '''
-    def __init__(self, dictionary_names=()) -> None:
+    def __init__(self, dictionary_names: Iterable[str] = ()) -> None:
         global DEBUG_LEVEL
         try:
-            DEBUG_LEVEL = int(str(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL')))
+            DEBUG_LEVEL = int(
+                str(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL')))
         except (TypeError, ValueError):
             DEBUG_LEVEL = int(0)
         if DEBUG_LEVEL > 1:
@@ -342,7 +357,7 @@ class Hunspell:
             else:
                 LOGGER.debug('Hunspell.__init__(dictionary_names=())\n')
         self._suggest_cache: Dict[str, List[Tuple[str, int]]] = {}
-        self._dictionary_names = dictionary_names
+        self._dictionary_names: List[str] = list(dictionary_names)
         self._dictionaries: List[Dictionary] = []
         self.init_dictionaries()
 
@@ -367,9 +382,9 @@ class Hunspell:
 
         It is important to return a copy, we do not want to change
         the private member variable directly.'''
-        return self._dictionary_names[:]
+        return list(self._dictionary_names[:])
 
-    def set_dictionary_names(self, dictionary_names: List[str]):
+    def set_dictionary_names(self, dictionary_names: List[str]) -> None:
         '''Sets the list of dictionary names.
 
         If the new list of dictionary names differs from the existing
@@ -457,8 +472,8 @@ class Hunspell:
         '''Return completions or corrections for the input phrase
 
         :param input_phrase: A string to find completions or corrections for
-        :type input_phrase: String
-        :rtype: A list of tuples of the form (<word>, <score>)
+
+        Returns a list of tuples of the form (<word>, <score>)
                 <score> can have these values:
                     0: This is a completion, i.e. input_phrase matches
                        the beginning of <word> (accent insensitive match)
@@ -504,8 +519,8 @@ class Hunspell:
         ('tenéis', 0)
 
         >>> h = Hunspell(['en_US'])
-        >>> h.suggest('camel')
-        [('camel', 0), ('camellia', 0), ('camelhair', 0), ('came', -1), ('Camel', -1), ('cameo', -1), ('came l', -1), ('camels', -1)]
+        >>> ('Camelot', 0) in h.suggest('camel') # Should work with aspell and hunspell
+        True
 
         >>> h = Hunspell(['fr_FR'])
         >>> h.suggest('differemmen')
@@ -516,9 +531,8 @@ class Hunspell:
         []
 
         >>> h = Hunspell(['None', 'en_US'])
-        >>> h.suggest('camel')
-        [('camel', 0), ('camellia', 0), ('camelhair', 0), ('came', -1), ('Camel', -1), ('cameo', -1), ('came l', -1), ('camels', -1)]
-
+        >>> ('Camelot', 0) in h.suggest('camel') # Should work with aspell and hunspell
+        True
         '''
         # pylint: enable=line-too-long
         if input_phrase in self._suggest_cache:
@@ -544,8 +558,6 @@ class Hunspell:
         # make sure input_phrase is in the internal normalization form (NFD):
         input_phrase = unicodedata.normalize(
             itb_util.NORMALIZATION_FORM_INTERNAL, input_phrase)
-        # But enchant and pyhunspell want NFC as input, make a copy in NFC:
-        input_phrase_nfc = unicodedata.normalize('NFC', input_phrase)
 
         suggested_words: Dict[str, int] = {}
         for dictionary in self._dictionaries:
@@ -559,15 +571,39 @@ class Hunspell:
                 # complete it, it just wastes time then.
                 if len(input_phrase) <= dictionary.max_word_len:
                     if dictionary.word_pairs:
-                        suggested_words.update([
-                            (x[0], 0)
-                            for x in dictionary.word_pairs
-                            if x[1].startswith(input_phrase_no_accents)])
+                        if IMPORT_REGEX_SUCCESFUL:
+                            regex_pattern = regex.compile(
+                                regex.escape(input_phrase_no_accents),
+                                regex.IGNORECASE)
+                            suggested_words.update([
+                                (x[0], 0)
+                                for x in dictionary.word_pairs
+                                if regex_pattern.match(x[1])])
+                        else:
+                            re_pattern = re.compile(
+                                re.escape(input_phrase_no_accents),
+                                re.IGNORECASE)
+                            suggested_words.update([
+                                (x[0], 0)
+                                for x in dictionary.word_pairs
+                                if re_pattern.match(x[1])])
                     else:
-                        suggested_words.update([
-                            (x, 0)
-                            for x in dictionary.words
-                            if x.startswith(input_phrase)])
+                        if IMPORT_REGEX_SUCCESFUL:
+                            regex_pattern = regex.compile(
+                                regex.escape(input_phrase),
+                                regex.IGNORECASE)
+                            suggested_words.update([
+                                (x, 0)
+                                for x in dictionary.words
+                                if regex_pattern.match(x)])
+                        else:
+                            re_pattern = re.compile(
+                                re.escape(input_phrase),
+                                re.IGNORECASE)
+                            suggested_words.update([
+                                (x, 0)
+                                for x in dictionary.words
+                                if re_pattern.match(x)])
                 if len(input_phrase) >= 4:
                     if dictionary.spellcheck(input_phrase):
                         # This is a valid word in this dictionary.
@@ -610,7 +646,7 @@ class Hunspell:
 
 BENCHMARK = True
 
-def main():
+def main() -> None:
     '''
     Used for testing and profiling.
 
@@ -618,17 +654,17 @@ def main():
 
     runs some tests and prints profiling data.
     '''
-    LOG_HANDLER = logging.StreamHandler(stream=sys.stderr)
+    log_handler = logging.StreamHandler(stream=sys.stderr)
     LOGGER.setLevel(logging.DEBUG)
-    LOGGER.addHandler(LOG_HANDLER)
+    LOGGER.addHandler(log_handler)
 
     if BENCHMARK:
-        import cProfile
-        import pstats
+        import cProfile # pylint: disable=import-outside-toplevel
+        import pstats # pylint: disable=import-outside-toplevel
         profile = cProfile.Profile()
         profile.enable()
 
-    import doctest
+    import doctest # pylint: disable=import-outside-toplevel
     (failed, dummy_attempted) = doctest.testmod()
 
     if BENCHMARK:
@@ -639,10 +675,10 @@ def main():
         stats.print_stats('hunspell', 25)
         stats.print_stats('enchant', 25)
 
-    if failed:
-        sys.exit(1)
-    else:
-        sys.exit(0)
+    LOGGER.info('itb_util.remove_accents() cache info: %s',
+                itb_util.remove_accents.cache_info())
+
+    sys.exit(failed)
 
 if __name__ == "__main__":
     main()
