@@ -10,7 +10,7 @@ echo "running $SCRIPT_DIR/mypy.sh"
 
 cd $SCRIPT_DIR/engine
 echo "Checking $SCRIPT_DIR/engine"
-mypy *.py
+mypy --strict *.py
 ENGINE_RETVAL=$?
 if [ ${ENGINE_RETVAL} != 0 ] ; then
     RETVAL=$((${RETVAL} + ${ENGINE_RETVAL}))
@@ -18,7 +18,7 @@ fi
 
 cd $SCRIPT_DIR/setup
 echo "Checking $SCRIPT_DIR/setup"
-mypy ../engine/{tabsqlitedb,itb_util,itb_emoji}.py *.py
+mypy --strict ../engine/{tabsqlitedb,itb_util,itb_emoji}.py *.py
 SETUP_RETVAL=$?
 if [ ${SETUP_RETVAL} != 0 ] ; then
     RETVAL=$((${RETVAL} + ${SETUP_RETVAL}))
@@ -26,7 +26,7 @@ fi
 
 cd $SCRIPT_DIR/tests
 echo "Checking $SCRIPT_DIR/tests"
-mypy ../engine/{tabsqlitedb,itb_util,itb_emoji}.py test_*.py
+mypy --strict ../engine/{tabsqlitedb,itb_util,itb_emoji}.py test_*.py
 SETUP_RETVAL=$?
 if [ ${SETUP_RETVAL} != 0 ] ; then
     RETVAL=$((${RETVAL} + ${SETUP_RETVAL}))
