@@ -19,12 +19,17 @@
 '''
 
 from typing import Any
+from typing import List
 from gi import require_version # type: ignore
 require_version('IBus', '1.0')
 from gi.repository import IBus # type: ignore
 
 class MockEngine:
-    def __init__(self, engine_name = '', connection = None, object_path = ''):
+    def __init__(
+            self,
+            engine_name: str = '',
+            connection: Any = None,
+            object_path: str = '') -> None:
         self.mock_auxiliary_text = ''
         self.mock_preedit_text = ''
         self.mock_preedit_text_cursor_pos = 0
@@ -43,10 +48,10 @@ class MockEngine:
         # engine does not try to support surrounding text, i.e.
         # we omit “| IBus.Capabilite.SURROUNDING_TEXT” here.
 
-    def update_auxiliary_text(self, text, visible):
+    def update_auxiliary_text(self, text: IBus.Text, visible: bool) -> None:
         self.mock_auxiliary_text = text.text
 
-    def commit_text(self, text):
+    def commit_text(self, text: IBus.Text) -> None:
         self.mock_committed_text = (
             self.mock_committed_text[
                 :self.mock_committed_text_cursor_pos]
@@ -55,7 +60,7 @@ class MockEngine:
                 self.mock_committed_text_cursor_pos:])
         self.mock_committed_text_cursor_pos += len(text.text)
 
-    def forward_key_event(self, val, code, state):
+    def forward_key_event(self, val: int, code: int, state: int) -> None:
         if (val == IBus.KEY_Left
             and self.mock_committed_text_cursor_pos > 0):
             self.mock_committed_text_cursor_pos -= 1
@@ -70,36 +75,46 @@ class MockEngine:
                 self.mock_committed_text_cursor_pos:])
             self.mock_committed_text_cursor_pos += len(unicode)
 
-    def update_lookup_table(self, table, visible):
+    def update_lookup_table(
+            self, table: IBus.LookupTable, visible: bool) -> None:
         pass
 
-    def update_preedit_text(self, text, cursor_pos, visible):
+    def update_preedit_text(self, text: IBus.Text, cursor_pos: int, visible: bool) -> None:
         self.mock_preedit_text = text.get_text()
         self.mock_preedit_text_cursor_pos = cursor_pos
         self.mock_preedit_text_visible = visible
 
     def update_preedit_text_with_mode(
-            self, text, cursor_pos, visible, focus_mode):
+            self,
+            text: IBus.Text,
+            cursor_pos: int,
+            visible: bool,
+            focus_mode: IBus.PreeditFocusMode) -> None:
         self.mock_preedit_focus_mode = focus_mode
         self.update_preedit_text(text, cursor_pos, visible)
 
-    def register_properties(self, property_list):
+    def register_properties(self, property_list: List[IBus.Property]) -> None:
         pass
 
-    def update_property(self, property):
+    def update_property(self, property: IBus.Property) -> None:
         pass
 
-    def hide_lookup_table(self):
+    def hide_lookup_table(self) -> None:
         pass
 
-    def get_surrounding_text(self):
+    def get_surrounding_text(self) -> None:
         pass
 
-    def connect(self, signal: str, callback_function: Any):
+    def connect(self, signal: str, callback_function: Any) -> None:
         pass
 
 class MockLookupTable:
-    def __init__(self, page_size = 9, cursor_pos = 0, cursor_visible = False, round = True):
+    def __init__(
+            self,
+            page_size: int = 9,
+            cursor_pos: int = 0,
+            cursor_visible: bool = False,
+            round: bool = True) -> None:
         self.clear()
         self.mock_labels = {
             1: '1.',
@@ -117,83 +132,83 @@ class MockLookupTable:
         self.mock_cursor_visible = cursor_visible
         self.cursor_visible = cursor_visible
         self.mock_round = round
-        self.mock_candidates = []
+        self.mock_candidates: List[str] = []
 
-    def clear(self):
+    def clear(self) -> None:
         self.mock_candidates = []
         self.mock_cursor_pos = 0
 
-    def set_label(self, index, label):
+    def set_label(self, index: int, label: IBus.Text) -> None:
         self.mock_labels[index] = label.get_text()
 
-    def get_label(self, index):
+    def get_label(self, index: int) -> str:
         return self.mock_labels[index]
 
-    def set_page_size(self, size):
+    def set_page_size(self, size: int) -> None:
         self.mock_page_size = size
 
-    def get_page_size(self):
+    def get_page_size(self) -> int:
         return self.mock_page_size
 
-    def set_round(self, round):
+    def set_round(self, round: bool) -> None:
         self.mock_round = round
 
-    def set_cursor_pos(self, pos):
+    def set_cursor_pos(self, pos: int) -> None:
         self.mock_cursor_pos = pos
 
-    def get_cursor_pos(self):
+    def get_cursor_pos(self) -> int:
         return self.mock_cursor_pos
 
-    def set_cursor_visible(self, visible):
+    def set_cursor_visible(self, visible: bool) -> None:
         self.mock_cursor_visible = visible
         self.cursor_visible = visible
 
-    def cursor_down(self):
+    def cursor_down(self) -> None:
         if len(self.mock_candidates):
             self.mock_cursor_pos += 1
             self.mock_cursor_pos %= len(self.mock_candidates)
 
-    def cursor_up(self):
+    def cursor_up(self) -> None:
         if len(self.mock_candidates):
             if self.mock_cursor_pos > 0:
                 self.mock_cursor_pos -= 1
             else:
                 self.mock_cursor_pos = len(self.mock_candidates) - 1
 
-    def set_orientation(self, orientation):
+    def set_orientation(self, orientation: int) -> None:
         self.mock_orientation = orientation
 
-    def get_number_of_candidates(self):
+    def get_number_of_candidates(self) -> int:
         return len(self.mock_candidates)
 
-    def append_candidate(self, candidate):
+    def append_candidate(self, candidate: IBus.Text) -> None:
         self.mock_candidates.append(candidate.get_text())
 
-    def get_candidate(self, index):
+    def get_candidate(self, index: int) -> str:
         return self.mock_candidates[index]
 
 class MockPropList:
-    def append(self, property):
+    def append(self, property: IBus.Property) -> None:
         pass
 
 class MockProperty:
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         pass
 
-    def set_label(self, ibus_text):
+    def set_label(self, ibus_text: IBus.Text) -> None:
         pass
 
-    def set_symbol(self, ibus_text):
+    def set_symbol(self, ibus_text: IBus.Text) -> None:
         pass
 
-    def set_tooltip(self, ibus_text):
+    def set_tooltip(self, ibus_text: IBus.Text) -> None:
         pass
 
-    def set_sensitive(self, sensitive):
+    def set_sensitive(self, sensitive: bool) -> None:
         pass
 
-    def set_visible(self, visible):
+    def set_visible(self, visible: bool) -> None:
         pass
 
-    def set_state(self, visible):
+    def set_state(self, visible: bool) -> None:
         pass
