@@ -37,17 +37,168 @@ import itb_util
 sys.path.pop(0)
 
 class ComposeSequencesTestCase(unittest.TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         locale.setlocale(locale.LC_CTYPE, 'en_US.UTF-8')
         self._compose_sequences = itb_util.ComposeSequences()
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         pass
 
-    def test_dummy(self):
+    def test_dummy(self) -> None:
         self.assertEqual(True, True)
 
-    def test_preedit_representations(self):
+    def test_adding_and_deleting_compose_sequences(self) -> None:
+        self._compose_sequences._add_compose_sequence(
+            '<Multi_key> <e> <m> <p> <t> <y>', '∅')
+        available_keyvals = None
+        keyvals = [IBus.KEY_Multi_key,
+                   IBus.KEY_e,
+                   IBus.KEY_m,
+                   IBus.KEY_p]
+        completions = self._compose_sequences.find_compose_completions(
+            keyvals, available_keyvals)
+        self.assertEqual(
+            ['ty'], self._compose_sequences._lookup_representations(completions))
+        keyvals = [IBus.KEY_Multi_key,
+                   IBus.KEY_e,
+                   IBus.KEY_m,
+                   IBus.KEY_p,
+                   IBus.KEY_t,
+                   IBus.KEY_y]
+        completions = self._compose_sequences.find_compose_completions(
+            keyvals, available_keyvals)
+        self.assertEqual(
+            [], self._compose_sequences._lookup_representations(completions))
+        self.assertEqual(
+            "∅", self._compose_sequences.compose(
+                [IBus.KEY_Multi_key,
+                 IBus.KEY_e,
+                 IBus.KEY_m,
+                 IBus.KEY_p,
+                 IBus.KEY_t,
+                 IBus.KEY_y]))
+        # Define a shorter compose sequence overriding the previous longer one:
+        self._compose_sequences._add_compose_sequence(
+            '<Multi_key> <e> <m> <p> <t>', '🕳️')
+        available_keyvals = None
+        keyvals = [IBus.KEY_Multi_key,
+                   IBus.KEY_e,
+                   IBus.KEY_m,
+                   IBus.KEY_p]
+        completions = self._compose_sequences.find_compose_completions(
+            keyvals, available_keyvals)
+        self.assertEqual(
+            ['t'], self._compose_sequences._lookup_representations(completions))
+        keyvals = [IBus.KEY_Multi_key,
+                   IBus.KEY_e,
+                   IBus.KEY_m,
+                   IBus.KEY_p,
+                   IBus.KEY_t]
+        completions = self._compose_sequences.find_compose_completions(
+            keyvals, available_keyvals)
+        self.assertEqual(
+            [], self._compose_sequences._lookup_representations(completions))
+        keyvals = [IBus.KEY_Multi_key,
+                   IBus.KEY_e,
+                   IBus.KEY_m,
+                   IBus.KEY_p,
+                   IBus.KEY_t,
+                   IBus.KEY_y]
+        completions = self._compose_sequences.find_compose_completions(
+            keyvals, available_keyvals)
+        self.assertEqual(
+            [], self._compose_sequences._lookup_representations(completions))
+        self.assertEqual(
+            "🕳️",
+            self._compose_sequences.compose(
+            [IBus.KEY_Multi_key,
+             IBus.KEY_e,
+             IBus.KEY_m,
+             IBus.KEY_p,
+             IBus.KEY_t]))
+        # The sequence '<Multi_key> <e> <m> <p> <t> <y> does not exist
+        # anymore now, but '<Multi_key> <e> <m> <p> <t>' already gives a result
+        # and the trailing '<y>' is ignored:
+        self.assertEqual(
+            "🕳️",
+            self._compose_sequences.compose(
+            [IBus.KEY_Multi_key,
+             IBus.KEY_e,
+             IBus.KEY_m,
+             IBus.KEY_p,
+             IBus.KEY_t,
+             IBus.KEY_y]))
+        # Now remove a compose sequence by using an empty replacement text:
+        self._compose_sequences._add_compose_sequence(
+            '<Multi_key> <e> <m> <p> <t> <y>', '')
+        available_keyvals = None
+        keyvals = [IBus.KEY_Multi_key,
+                   IBus.KEY_e,
+                   IBus.KEY_m,
+                   IBus.KEY_p]
+        completions = self._compose_sequences.find_compose_completions(
+            keyvals, available_keyvals)
+        self.assertEqual(
+            [], self._compose_sequences._lookup_representations(completions))
+        keyvals = [IBus.KEY_Multi_key,
+                   IBus.KEY_e,
+                   IBus.KEY_m,
+                   IBus.KEY_p,
+                   IBus.KEY_t]
+        completions = self._compose_sequences.find_compose_completions(
+            keyvals, available_keyvals)
+        self.assertEqual(
+            [], self._compose_sequences._lookup_representations(completions))
+        keyvals = [IBus.KEY_Multi_key,
+                   IBus.KEY_e,
+                   IBus.KEY_m,
+                   IBus.KEY_p,
+                   IBus.KEY_t,
+                   IBus.KEY_y]
+        completions = self._compose_sequences.find_compose_completions(
+            keyvals, available_keyvals)
+        self.assertEqual(
+            [], self._compose_sequences._lookup_representations(completions))
+        # Invalid now, sequence has been deleted (invalid sequence returns
+        # and empty string):
+        self.assertEqual(
+            '',
+            self._compose_sequences.compose(
+            [IBus.KEY_Multi_key,
+             IBus.KEY_e,
+             IBus.KEY_m,
+             IBus.KEY_p,
+             IBus.KEY_t,
+             IBus.KEY_y]))
+        self.assertEqual(
+            '',
+            self._compose_sequences.compose(
+            [IBus.KEY_Multi_key,
+             IBus.KEY_e,
+             IBus.KEY_m,
+             IBus.KEY_p]))
+        # Check that there are still sequences starting with '<Multi_key> <e>'
+        # they should not have been deleted:
+        available_keyvals = None
+        keyvals = [IBus.KEY_Multi_key,
+                   IBus.KEY_e]
+        completions = self._compose_sequences.find_compose_completions(
+            keyvals, available_keyvals)
+        self.assertNotEqual(
+            [], self._compose_sequences._lookup_representations(completions))
+        self.assertEqual(
+            None,
+            self._compose_sequences.compose(
+            [IBus.KEY_Multi_key,
+             IBus.KEY_e]))
+        self.assertEqual(
+            '€',
+            self._compose_sequences.compose(
+            [IBus.KEY_Multi_key,
+             IBus.KEY_e,
+             IBus.KEY_equal]))
+
+    def test_preedit_representations(self) -> None:
         self.assertEqual(
             self._compose_sequences.preedit_representation(
                 [IBus.KEY_Multi_key,
@@ -80,7 +231,7 @@ class ComposeSequencesTestCase(unittest.TestCase):
                 [IBus.KEY_a, IBus.KEY_dead_belowdiaeresis]),
             'a\u00A0\u0324')
 
-    def test_compose(self):
+    def test_compose(self) -> None:
         # Valid sequence should return a composed result:
         self.assertEqual(
             self._compose_sequences.compose(
@@ -127,7 +278,7 @@ class ComposeSequencesTestCase(unittest.TestCase):
                  IBus.KEY_quotedbl]),
             'ä')
 
-    def test_compose_parse_double_quote_in_result(self):
+    def test_compose_parse_double_quote_in_result(self) -> None:
         # Make sure this sequence from
         # /usr/share/X11/locale/en_US.UTF-8/Compose is parsed
         # correctly:
@@ -139,7 +290,7 @@ class ComposeSequencesTestCase(unittest.TestCase):
                  IBus.KEY_space]),
             '"')
 
-    def test_compose_cs_CZ(self):
+    def test_compose_cs_CZ(self) -> None:
         # /usr/share/X11/locale/cs_CZ.UTF-8/Compose overrides some of
         # the compose sequences from
         # /usr/share/X11/locale/en_US.UTF-8/Compose:
@@ -165,7 +316,7 @@ class ComposeSequencesTestCase(unittest.TestCase):
                  IBus.KEY_quotedbl]),
             'ä')
 
-    def test_compose_km_KH(self):
+    def test_compose_km_KH(self) -> None:
         locale.setlocale(locale.LC_CTYPE, 'km_KH.UTF-8')
         self._compose_sequences = itb_util.ComposeSequences()
         # This sequence comes from
@@ -190,7 +341,7 @@ class ComposeSequencesTestCase(unittest.TestCase):
                 [0x17FF]),
             'ាំ')
 
-    def test_compose_pt_BR(self):
+    def test_compose_pt_BR(self) -> None:
         # This sequence comes from
         # /usr/share/X11/locale/en_US.UTF-8/Compose and is not
         # overridden in /usr/share/X11/locale/pt_BR.UTF-8/:
@@ -279,7 +430,7 @@ class ComposeSequencesTestCase(unittest.TestCase):
                  IBus.KEY_quotedbl]),
             'ä')
 
-    def test_compose_pt_PT(self):
+    def test_compose_pt_PT(self) -> None:
         # These sequences come from
         # /usr/share/X11/locale/en_US.UTF-8/Compose and is not
         # overridden in /usr/share/X11/locale/pt_PT.UTF-8/:
@@ -337,7 +488,7 @@ class ComposeSequencesTestCase(unittest.TestCase):
                  IBus.KEY_comma]),
             'Ų')
 
-    def test_compose_am_ET(self):
+    def test_compose_am_ET(self) -> None:
         # These sequences come from
         # /usr/share/X11/locale/en_US.UTF-8/Compose:
         self.assertEqual(
@@ -386,7 +537,7 @@ class ComposeSequencesTestCase(unittest.TestCase):
                  IBus.KEY_quotedbl]),
             '')
 
-    def test_reasonable_dead_key_sequences(self):
+    def test_reasonable_dead_key_sequences(self) -> None:
         # “reasonable” dead key sequences are are handled even if they
         # are not defined in any Compose file.
         self.assertEqual(
@@ -407,7 +558,7 @@ class ComposeSequencesTestCase(unittest.TestCase):
                  IBus.KEY_e]),
             '\u0117\u0304') # ė̄
 
-    def test_keypad_fallback(self):
+    def test_keypad_fallback(self) -> None:
         self.assertEqual(
             None,
             self._compose_sequences.compose(
