@@ -28,6 +28,8 @@ from gi import require_version # type: ignore
 require_version('IBus', '1.0')
 from gi.repository import IBus # type: ignore
 
+import testutils
+
 sys.path.insert(0, "../engine")
 import hunspell_suggest
 import itb_util
@@ -52,48 +54,14 @@ try:
 except (ImportError,):
     pass
 
-def get_libvoikko_version():
-    if IMPORT_LIBVOIKKO_SUCCESSFUL:
-        return libvoikko.Voikko.getVersion()
-    else:
-        return '0'
-
-def enchant_sanity_test(language: str = '', word: str = '') -> bool:
-    '''Checks whether python3-enchant returns some suggestions given a
-    language and a word.
-
-    :param language: The language of the dictionary to try
-    :param word: The word to give to enchant to ask for suggestions
-
-    This is used as a sanity check whether python3-enchant works at all.
-    For example, if a Czech dictionary is opened like
-
-        d = enchant.Dict('cs_CZ')
-
-    and then something like
-
-        retval = d.suggest('Praha')
-
-    returns an empty list instead of a list of some words, then
-    something is seriously wrong with python3-enchant and it is better
-    to skip the test case which relies on python3-enchant working for
-    that language.
-    '''
-    if not (language and word):
-        return False
-    d = enchant.Dict(language)
-    if d.suggest(word):
-        return True
-    return  False
-
 class HunspellSuggestTestCase(unittest.TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.maxDiff = None
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         pass
 
-    def test_dummy(self):
+    def test_dummy(self) -> None:
         self.assertEqual(True, True)
 
     @unittest.skipUnless(
@@ -106,9 +74,9 @@ class HunspellSuggestTestCase(unittest.TestCase):
         itb_util.get_hunspell_dictionary_wordlist('de_DE')[0],
         'Skipping because no German hunspell dictionary could be found.')
     @unittest.skipUnless(
-        enchant_sanity_test(language='cs_CZ', word='Praha'),
+        testutils.enchant_sanity_test(language='cs_CZ', word='Praha'),
         'Skipping because python3-enchant seems broken for cs_CZ.')
-    def test_de_DE_cs_CZ_enchant(self):
+    def test_de_DE_cs_CZ_enchant(self) -> None:
         h = hunspell_suggest.Hunspell(['de_DE', 'cs_CZ'])
         self.assertEqual(
             h.suggest('Geschwindigkeitsubertre')[0],
@@ -145,7 +113,7 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('de_DE')[0],
         'Skipping because no German hunspell dictionary could be found.')
-    def test_de_DE_cs_CZ_pyhunspell(self):
+    def test_de_DE_cs_CZ_pyhunspell(self) -> None:
         h = hunspell_suggest.Hunspell(['de_DE', 'cs_CZ'])
         self.assertEqual(
             h.suggest('Geschwindigkeitsubertre')[0],
@@ -176,7 +144,7 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('it_IT')[0],
         'Skipping because no Italian hunspell dictionary could be found.')
-    def test_it_IT(self):
+    def test_it_IT(self) -> None:
         h = hunspell_suggest.Hunspell(['it_IT'])
         self.assertEqual(
             h.suggest('principianti'),
@@ -189,7 +157,7 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('es_ES')[0],
         'Skipping because no Spanish hunspell dictionary could be found.')
-    def test_es_ES(self):
+    def test_es_ES(self) -> None:
         h = hunspell_suggest.Hunspell(['es_ES'])
         self.assertEqual(
             h.suggest('teneis'),
@@ -207,23 +175,58 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('en_US')[0],
         'Skipping because no US English hunspell dictionary could be found.')
-    def test_en_US(self):
+    @unittest.skipUnless(
+        testutils.enchant_sanity_test(language='cs_CZ', word='Praha'),
+        'Skipping because python3-enchant seems broken for cs_CZ.')
+    @unittest.skipUnless(
+        testutils.enchant_working_as_expected(),
+        'Skipping because of an unexpected change in the enchant behaviour.')
+    def test_en_US(self) -> None:
         h = hunspell_suggest.Hunspell(['en_US'])
         self.assertEqual(
-            h.suggest('camel'),
-            [('camel', 0),
+            [('Camel', 0),
+             ('camel', 0),
+             ('Camelot', 0),
              ('camellia', 0),
              ('camelhair', 0),
+             ('Camelopardalis', 0),
+             ('CAM', -1),
+             ('Cal', -1),
+             ('Mel', -1),
+             ('cal', -1),
+             ('cam', -1),
+             ('Carl', -1),
+             ('Gael', -1),
+             ('Jame', -1),
+             ('call', -1),
              ('came', -1),
-             ('Camel', -1),
+             ('come', -1),
+             ('game', -1),
+             ('Jamal', -1),
+             ('Jamel', -1),
+             ('Ocaml', -1),
+             ('cable', -1),
              ('cameo', -1),
-             ('came l', -1),
-             ('camels', -1)])
+             ('calmer', -1),
+             ('camels', -1),
+             ('comely', -1),
+             ('compel', -1),
+             ('gamely', -1),
+             ("Camel's", -1),
+             ('Camilla', -1),
+             ('Camille', -1),
+             ('Carmela', -1),
+             ('Carmelo', -1),
+             ("Jamel's", -1),
+             ("camel's", -1),
+             ('caramel', -1),
+             ('Carmella', -1)],
+            h.suggest('camel'))
 
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('fr_FR')[0],
         'Skipping because no French hunspell dictionary could be found.')
-    def test_fr_FR(self):
+    def test_fr_FR(self) -> None:
         h = hunspell_suggest.Hunspell(['fr_FR'])
         self.assertEqual(
             h.suggest('differemmen'),
@@ -232,13 +235,13 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('el_GR')[0],
         "Skipping because no Greek dictionary could be found. ")
-    def test_el_GR(self):
+    def test_el_GR(self) -> None:
         h = hunspell_suggest.Hunspell(['el_GR'])
         self.assertEqual(
             h.suggest('αλφαβητο')[0],
             ('αλφάβητο', 0))
 
-    def test_fi_FI_dictionary_file(self):
+    def test_fi_FI_dictionary_file(self) -> None:
         # dictionary file is included in ibus-typing-booster
         h = hunspell_suggest.Hunspell(['fi_FI'])
         self.assertEqual(
@@ -251,9 +254,9 @@ class HunspellSuggestTestCase(unittest.TestCase):
              ('kissamaiseksi',0)])
 
     @unittest.skipUnless(
-        get_libvoikko_version() >= '4.3',
+        testutils.get_libvoikko_version() >= '4.3',
         "Skipping, requires python3-libvoikko version >= 4.3.")
-    def test_fi_FI_voikko(self):
+    def test_fi_FI_voikko(self) -> None:
         d = hunspell_suggest.Dictionary('fi_FI')
         self.assertEqual(d.has_spellchecking(), True)
         h = hunspell_suggest.Hunspell(['fi_FI'])
@@ -276,7 +279,7 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('en_US')[0],
         'Skipping because no US English hunspell dictionary could be found.')
-    def test_en_US_spellcheck_enchant(self):
+    def test_en_US_spellcheck_enchant(self) -> None:
         d = hunspell_suggest.Dictionary('en_US')
         self.assertEqual(d.spellcheck_enchant('winter'), True)
         self.assertEqual(d.spellcheck_enchant('winxer'), False)
@@ -287,11 +290,17 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('en_US')[0],
         'Skipping because no US English hunspell dictionary could be found.')
-    def test_en_US_spellcheck_suggest_enchant(self):
+    @unittest.skipUnless(
+        testutils.enchant_sanity_test(language='cs_CZ', word='Praha'),
+        'Skipping because python3-enchant seems broken for cs_CZ.')
+    @unittest.skipUnless(
+        testutils.enchant_working_as_expected(),
+        'Skipping because of an unexpected change in the enchant behaviour.')
+    def test_en_US_spellcheck_suggest_enchant(self) -> None:
         d = hunspell_suggest.Dictionary('en_US')
         self.assertEqual(
             d.spellcheck_suggest_enchant('kamel'),
-            ['camel', 'Camel'])
+            ['Jamel', 'Camel', 'camel', 'Jamal', 'gamely'])
 
     @unittest.skipUnless(
         IMPORT_HUNSPELL_SUCCESSFUL and not IMPORT_ENCHANT_SUCCESSFUL,
@@ -299,7 +308,7 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('en_US')[0],
         'Skipping because no US English hunspell dictionary could be found.')
-    def test_en_US_spellcheck_pyhunspell(self):
+    def test_en_US_spellcheck_pyhunspell(self) -> None:
         d = hunspell_suggest.Dictionary('en_US')
         self.assertEqual(d.spellcheck_pyhunspell('winter'), True)
         self.assertEqual(d.spellcheck_pyhunspell('winxer'), False)
@@ -310,16 +319,16 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('en_US')[0],
         'Skipping because no US English hunspell dictionary could be found.')
-    def test_en_US_spellcheck_suggest_pyhunspell(self):
+    def test_en_US_spellcheck_suggest_pyhunspell(self) -> None:
         d = hunspell_suggest.Dictionary('en_US')
         self.assertEqual(
             d.spellcheck_suggest_pyhunspell('kamel'),
             ['camel', 'Camel'])
 
     @unittest.skipUnless(
-        get_libvoikko_version() >= '4.3',
+        testutils.get_libvoikko_version() >= '4.3',
         "Skipping, requires python3-libvoikko version >= 4.3.")
-    def test_fi_FI_spellcheck_voikko(self):
+    def test_fi_FI_spellcheck_voikko(self) -> None:
         d = hunspell_suggest.Dictionary('fi_FI')
         self.assertEqual(d.spellcheck_voikko('kissa'), True)
         self.assertEqual(d.spellcheck_voikko('kisssa'), False)
@@ -327,9 +336,9 @@ class HunspellSuggestTestCase(unittest.TestCase):
         self.assertEqual(d.spellcheck_voikko('Päivia'), False)
 
     @unittest.skipUnless(
-        get_libvoikko_version() >= '4.3',
+        testutils.get_libvoikko_version() >= '4.3',
         "Skipping, requires python3-libvoikko version >= 4.3.")
-    def test_fi_FI_spellcheck_suggest_voikko(self):
+    def test_fi_FI_spellcheck_suggest_voikko(self) -> None:
         d = hunspell_suggest.Dictionary('fi_FI')
         self.assertEqual(
             d.spellcheck_suggest_voikko('kisssa'),
@@ -338,11 +347,12 @@ class HunspellSuggestTestCase(unittest.TestCase):
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('sv_SE')[0],
         "Skipping because no Swedisch dictionary could be found. ")
-    def test_sv_SE(self):
+    def test_sv_SE(self) -> None:
         h = hunspell_suggest.Hunspell(['sv_SE'])
         self.assertEqual(
             h.suggest('östgo'),
             [('östgot', 0),
+             ('Östgöta', 0),
              ('östgöte', 0),
              ('östgotisk', 0),
              ('östgötsk', 0),
@@ -354,12 +364,14 @@ class HunspellSuggestTestCase(unittest.TestCase):
              ('Östgot', -1)])
         self.assertEqual(
             h.suggest('östgö'),
-            [('östgöte', 0),
+            [('Östgöta', 0),
+             ('östgöte', 0),
              ('östgötsk', 0),
              ('östgötska', 0)])
         self.assertEqual(
-            h.suggest('östgöt')[0:4],
-            [('östgöte', 0),
+            h.suggest('östgöt')[0:5],
+            [('Östgöta', 0),
+             ('östgöte', 0),
              ('östgötsk', 0),
              ('östgötska', 0),
              ('östgot', -1)])
