@@ -40,18 +40,32 @@ try:
 except (ImportError,):
     IMPORT_DISTRO_SUCCESSFUL = False
 
+IMPORT_LANGTABLE_SUCCESSFUL = False
+try:
+    import langtable # type: ignore
+    IMPORT_LANGTABLE_SUCCESSFUL = True
+except (ImportError,):
+    IMPORT_LANGTABLE_SUCCESSFUL = False
+
+IMPORT_PYCOUNTRY_SUCCESSFUL = False
+try:
+    import pycountry # type: ignore
+    IMPORT_PYCOUNTRY_SUCCESSFUL = True
+except (ImportError,):
+    IMPORT_PYCOUNTRY_SUCCESSFUL = False
+
 sys.path.insert(0, "../engine")
 import itb_util
 sys.path.pop(0)
 
 class ItbUtilTestCase(unittest.TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         pass
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         pass
 
-    def test_dummy(self):
+    def test_dummy(self) -> None:
         self.assertEqual(True, True)
 
     @unittest.skipUnless(
@@ -60,7 +74,7 @@ class ItbUtilTestCase(unittest.TestCase):
         and distro.version() >= '34',
         'Skipping, requires new enough m17n-db package, '
         'might not be available on older distributions.')
-    def test_default_input_methods_available(self):
+    def test_default_input_methods_available(self) -> None:
         m17n_db_info = itb_util.M17nDbInfo()
         available_imes = m17n_db_info.get_imes()
         missing_imes_for_defaults = []
@@ -70,7 +84,37 @@ class ItbUtilTestCase(unittest.TestCase):
                     missing_imes_for_defaults.append(ime)
         self.assertEqual([], missing_imes_for_defaults)
 
-    def test_is_right_to_left_messages(self):
+    @unittest.skipUnless(
+        IMPORT_LANGTABLE_SUCCESSFUL or IMPORT_PYCOUNTRY_SUCCESSFUL,
+        'Skipping, requires langtable or pycountry.')
+    def test_locale_text_to_match(self) -> None:
+        if 'LC_ALL' in os.environ:
+            del os.environ['LC_ALL']
+        os.environ['LC_MESSAGES'] = 'de_DE.UTF-8'
+        text_to_match = itb_util.locale_text_to_match('fr_FR')
+        if IMPORT_LANGTABLE_SUCCESSFUL:
+            self.assertEqual(
+                'fr_fr franzosisch (frankreich) francais (france) french (france)',
+                text_to_match)
+        elif IMPORT_PYCOUNTRY_SUCCESSFUL:
+            self.assertEqual(
+                'fr_fr french franzosisch francais france frankreich france',
+                text_to_match)
+
+    @unittest.skipUnless(
+        IMPORT_LANGTABLE_SUCCESSFUL or IMPORT_PYCOUNTRY_SUCCESSFUL,
+        'Skipping, requires langtable or pycountry.')
+    def test_locale_language_description(self) -> None:
+        if 'LC_ALL' in os.environ:
+            del os.environ['LC_ALL']
+        os.environ['LC_MESSAGES'] = 'de_DE.UTF-8'
+        language_description = itb_util.locale_language_description('fr_FR')
+        if IMPORT_LANGTABLE_SUCCESSFUL or IMPORT_PYCOUNTRY_SUCCESSFUL:
+            self.assertEqual(
+                'Französisch (Frankreich)',
+                language_description)
+
+    def test_is_right_to_left_messages(self) -> None:
         if 'LC_ALL' in os.environ:
             del os.environ['LC_ALL']
         os.environ['LC_MESSAGES'] = 'de_DE.UTF-8'
@@ -91,7 +135,7 @@ class ItbUtilTestCase(unittest.TestCase):
         os.environ['LC_ALL'] = 'C'
         self.assertEqual(itb_util.is_right_to_left_messages(), False)
 
-    def test_remove_accents(self):
+    def test_remove_accents(self) -> None:
         self.assertEqual(
             itb_util.remove_accents('abcÅøßẞüxyz'),
             'abcAossSSuxyz')
